@@ -140,12 +140,13 @@ class Artifacts(object):
     @classmethod
     def get_artifacts_list(self):
         return {'server': r'OMERO\.server.*\.zip',
-                'source': r'OMERO\.source.*\.zip',
+                'source': r'openmicroscopy.*\.zip',
                 'win': r'OMERO\.clients.*\.win\.zip',
                 'linux': r'OMERO\.clients.*\.linux\.zip',
                 'mac': r'OMERO\.clients.*\.mac\.zip',
                 'matlab': r'OMERO\.matlab.*\.zip',
                 'cpp': r'OMERO\.cpp.*\.zip',
+                'python': r'OMERO\.py.*\.zip',
                 }
 
     def download(self, component):
