@@ -3,12 +3,9 @@
 
 import os
 import shutil
-import subprocess
+import tempfile
 import logging
 
-import fileinput
-import smtplib
-
 from artifacts import Artifacts
 from db import DbAdmin
 from external import External
@@ -16,40 +13,10 @@ from yaclifw.framework import Command, Stop
 import fileutils
 from env import EnvDefault, DbParser, FileUtilsParser, JenkinsParser
 from env import WINDOWS
-from env import HOSTNAME
 
 log = logging.getLogger("omego.upgrade")
 
 
-class Email(object):
-
-    def __init__(self, artifacts, args):
-
-        TO = args.recipients
-        FROM = args.sender
-        text = "The OMERO.server on %s has been upgraded. \n" \
-               "=========================\n" \
-               "THIS SERVER REQUIRES VPN!\n" \
-               "=========================\n" \
-               "Please download suitable clients from: \n " \
-               "\n - Windows: \n %s\n " \
-               "\n - MAC: \n %s\n " \
-               "\n - Linux: \n %s\n " \
-               "\n - Webclient available on %s. \n \n " %\
-               (args.server, artifacts.win, artifacts.mac,
-                artifacts.linux, args.weburl)
-        BODY = "\r\n".join(("From: %s" % FROM,
-                            "To: %s" % TO,
-                            "Subject: %s" % args.subject,
-                            "",
-                            text))
-        server = smtplib.SMTP(args.smtp_server)
-        server.sendmail(FROM, args.recipients, BODY)
-        server.quit()
-
-        log.info("Mail was sent to: %s", args.recipients)
-
-
 class Install(object):
 
     def __init__(self, cmd, args):
@@ -74,10 +41,10 @@ class Install(object):
         if newinstall:
             # Create a symlink to simplify the rest of the logic-
             # just need to check if OLD == NEW
-            self.mklink(server_dir)
-            log.info("Upgrading %s (%s)...", server_dir, args.sym)
-        else:
+            self.symlink(server_dir, args.sym)
             log.info("Installing %s (%s)...", server_dir, args.sym)
+        else:
+            log.info("Upgrading %s (%s)...", server_dir, args.sym)
 
         self.external = External(server_dir)
         self.external.setup_omero_cli()
@@ -87,13 +54,13 @@ class Install(object):
 
         # Need lib/python set above
         import path
-        self.cfg = path.path(args.cfg)
         self.dir = path.path(server_dir)
 
         if not newinstall:
             self.stop()
 
-        self.configure(self.external.has_config())
+        copyold = not newinstall and not args.ignoreconfig
+        self.configure(copyold, args.prestartfile)
         self.directories()
 
         if newinstall:
@@ -143,39 +110,50 @@ class Install(object):
             log.error('Error whilst stopping server: %s', e)
 
         if self.web():
-            log.info("Stopping web")
-            self.stopweb()
+            try:
+                log.info("Stopping web")
+                self.stopweb()
+            except Exception as e:
+                log.error('Error whilst stopping web: %s', e)
 
-    def configure(self, noconfigure):
+    def configure(self, copyold, prestartfile):
+        def samecontents(a, b):
+            # os.path.samefile is not available on Windows
+            try:
+                return os.path.samefile(a, b)
+            except AttributeError:
+                with open(a) as fa:
+                    with open(b) as fb:
+                        return fa.read() == fb.read()
 
         target = self.dir / "etc" / "grid" / "config.xml"
-        if noconfigure:
-            log.warn("Target %s already exists, skipping.", target)
-            self.configure_ports()
-            return  # Early exit!
 
-        if not self.cfg.exists():
-            log.info("%s not found. Copying old files", self.cfg)
+        if copyold:
             from path import path
             old_grid = path(self.args.sym) / "etc" / "grid"
             old_cfg = old_grid / "config.xml"
-            if target.exists() and os.path.samefile(old_cfg, target):
+            log.info("Copying old configuration from %s", old_cfg)
+            if not old_cfg.exists():
+                raise Stop(40, 'config.xml not found')
+            if target.exists() and samecontents(old_cfg, target):
                 # This likely is caused by the symlink being
                 # created early on an initial install.
                 pass
             else:
                 old_cfg.copy(target)
         else:
-            self.cfg.copy(target)
-            # TODO: Unneeded if copy old?
-            self.run(["config", "set", "omero.web.server_list", self.args.web])
+            if target.exists():
+                log.info('Deleting configuration file %s', target)
+                target.remove()
 
-        log.debug('Configuring JVM memory')
-        templates = self.dir / "etc" / "grid" / "templates.xml"
-        for line in fileinput.input([templates], inplace=True):
-            line = line.replace("Xmx512M", self.args.mem)
-            line = line.replace("Xmx256M", self.args.mem)
-            print line,
+        if prestartfile:
+            for f in prestartfile:
+                log.info('Loading prestart file %s', f)
+                ftype, fpath = fileutils.get_as_local_path(f, 'backup')
+                if ftype != 'file':
+                    raise Stop(50, 'Expected file, found: %s %s' % (
+                        ftype, f))
+                self.run(['load', fpath])
 
         self.configure_ports()
 
@@ -185,6 +163,36 @@ class Install(object):
                  self.args.registry, "--tcp",
                  self.args.tcp, "--ssl", self.args.ssl])
 
+    def directories(self):
+        if self.samedir(self.dir, self.args.sym):
+            log.warn("Upgraded server was the same, not deleting")
+            return
+
+        try:
+            target = self.readlink(self.args.sym)
+            targetzip = target + '.zip'
+        except IOError:
+            log.error('Unable to get symlink target: %s', self.args.sym)
+            target = None
+            targetzip = None
+
+        if "false" == self.args.skipdelete.lower() and target:
+            try:
+                log.info("Deleting %s", target)
+                shutil.rmtree(target)
+            except OSError as e:
+                log.error("Failed to delete %s: %s", target, e)
+
+        if "false" == self.args.skipdeletezip.lower() and targetzip:
+            try:
+                log.info("Deleting %s", targetzip)
+                os.unlink(targetzip)
+            except OSError as e:
+                log.error("Failed to delete %s: %s", targetzip, e)
+
+        self.rmlink(self.args.sym)
+        self.symlink(self.dir, self.args.sym)
+
     def init_db(self):
         if self.args.initdb:
             log.debug('Initialising database')
@@ -233,41 +241,25 @@ class UnixInstall(Install):
     def startweb(self):
         self.run("web start")
 
-    def directories(self):
-        if os.path.samefile(self.dir, self.args.sym):
-            log.warn("Upgraded server was the same, not deleting")
-            return
+    def samedir(self, targetdir, link):
+        return os.path.samefile(targetdir, link)
 
-        target = os.readlink(self.args.sym)
-        # normpath in case there's a trailing /
-        targetzip = os.path.normpath(target) + '.zip'
-
-        if "false" == self.args.skipdelete.lower():
-            try:
-                log.info("Deleting %s", target)
-                shutil.rmtree(target)
-            except OSError as e:
-                log.error("Failed to delete %s: %s", target, e)
-
-        if "false" == self.args.skipdeletezip.lower():
-            try:
-                log.info("Deleting %s", targetzip)
-                os.unlink(targetzip)
-            except OSError as e:
-                log.error("Failed to delete %s: %s", targetzip, e)
+    def readlink(self, link):
+        return os.path.normpath(os.readlink(link))
 
+    def rmlink(self, link):
         try:
-            os.unlink(self.args.sym)
+            os.unlink(link)
         except OSError as e:
-            log.error("Failed to unlink %s: %s", self.args.sym, e)
+            log.error("Failed to unlink %s: %s", link, e)
+            raise
 
-        self.mklink(self.dir)
-
-    def mklink(self, dir):
+    def symlink(self, targetdir, link):
         try:
-            os.symlink(dir, self.args.sym)
+            os.symlink(targetdir, link)
         except OSError as e:
-            log.error("Failed to symlink %s to %s: %s", dir, self.args.sym, e)
+            log.error("Failed to symlink %s to %s: %s", targetdir, link, e)
+            raise
 
 
 class WindowsInstall(Install):
@@ -282,32 +274,64 @@ class WindowsInstall(Install):
         self.run("web iis")
         self.iisreset()
 
-    def directories(self):
-        self.rmdir()  # TODO: skipdelete etc?
-        log.warn(
-            "Should probably move directory to OLD_OMERO and test handles")
-        self.mklink(self.dir)
+    # os.path.samefile doesn't work on Python 2
+    # Create a tempfile in one directory and test for it's existence in the
+    # other
 
-    def call(self, command):
-        rc = subprocess.call(command, shell=True)
-        if rc != 0:
-            log.warn("'%s' returned with non-zero value: %s", command, rc)
+    def samedir(self, targetdir, link):
+        try:
+            return os.path.samefile(targetdir, link)
+        except AttributeError:
+            with tempfile.NamedTemporaryFile(dir=targetdir) as test:
+                return os.path.exists(
+                    os.path.join(link, os.path.basename(test.name)))
 
-    def rmdir(self):
-        """
-        """
-        self.call("rmdir %s".split() % self.args.sym)
+    # Symlinks are a bit more complicated on Windows:
+    # - You must have (elevated) administrator privileges
+    # - os.symlink doesn't work on Python 2, you must use a win32 call
+    # - os.readlink doesn't work on Python 2, and the solution suggested in
+    #   http://stackoverflow.com/a/7924557 doesn't work for me.
+    #
+    # We need to dereference the symlink in order to delete the old server
+    # so for now just store it in a text file alongside the symlink.
 
-    def mklink(self, dir):
+    def readlink(self, link):
+        try:
+            return os.path.normpath(os.readlink(link))
+        except AttributeError:
+            with open('%s.target' % link, 'r') as f:
+                return os.path.normpath(f.read())
+
+    def rmlink(self, link):
         """
         """
-        self.call("mklink /d %s".split() % self.args.sym + ["%s" % dir])
+        if os.path.isdir(link):
+            os.rmdir(link)
+        else:
+            os.unlink(link)
+
+    def symlink(self, targetdir, link):
+        """
+        """
+        try:
+            os.symlink(targetdir, link)
+        except AttributeError:
+            import win32file
+            flag = 1 if os.path.isdir(targetdir) else 0
+            try:
+                win32file.CreateSymbolicLink(link, targetdir, flag)
+            except Exception as e:
+                log.error(
+                    "Failed to symlink %s to %s: %s", targetdir, link, e)
+                raise
+            with open('%s.target' % link, 'w') as f:
+                f.write(targetdir)
 
     def iisreset(self):
         """
         Calls iisreset
         """
-        self.call(["iisreset"])
+        self.external.run('iisreset', [])
 
 
 class InstallBaseCommand(Command):
@@ -319,36 +343,24 @@ class InstallBaseCommand(Command):
     def __init__(self, sub_parsers):
         super(InstallBaseCommand, self).__init__(sub_parsers)
 
-        # TODO: these are very internal values and should be refactored out
-        # to a configure file.
-        skipemail = "false"
-        name = HOSTNAME
-        if HOSTNAME == "gretzky":
-            address = "gretzky.openmicroscopy.org.uk"
-        elif HOSTNAME == "howe":
-            address = "howe.openmicroscopy.org.uk"
-        elif HOSTNAME == "ome-dev-svr":
-            name = "win-2k8"
-            address = "bp.openmicroscopy.org.uk"
-        else:
-            address = HOSTNAME
-            # Don't send emails if we're not on a known host.
-            skipemail = "true"
-
         self.parser.add_argument("-n", "--dry-run", action="store_true")
         self.parser.add_argument(
             "server", nargs="?", help="The server directory, or a server-zip, "
             "or the url of a server-zip")
 
+        self.parser.add_argument(
+            "--prestartfile", action="append",
+            help="Run these OMERO commands before starting server, "
+                 "can be repeated")
+        self.parser.add_argument(
+            "--ignoreconfig", action="store_true",
+            help="Don't copy the old configuration file when upgrading")
+
         self.parser = JenkinsParser(self.parser)
         self.parser = DbParser(self.parser)
         self.parser = FileUtilsParser(self.parser)
 
         Add = EnvDefault.add
-        Add(self.parser, "hostname", HOSTNAME)
-        Add(self.parser, "name", name)
-        Add(self.parser, "address", address)
-        Add(self.parser, "skipemail", skipemail)
 
         # Ports
         Add(self.parser, "prefix", "")
@@ -356,26 +368,7 @@ class InstallBaseCommand(Command):
         Add(self.parser, "tcp", "%(prefix)s4063")
         Add(self.parser, "ssl", "%(prefix)s4064")
 
-        # new_server.py
-        cfg = os.path.join(os.path.expanduser("~"), "config.xml")
-        Add(self.parser, "mem", "Xmx1024M")
         Add(self.parser, "sym", "OMERO-CURRENT")
-        Add(self.parser, "cfg", cfg)
-
-        web = """[["localhost", %(ssl)s, "%(name)s"]"""
-        web += """, ["gretzky.openmicroscopy.org.uk", 4064, "gretzky"]"""
-        web += """, ["howe.openmicroscopy.org.uk", 4064, "howe"]]"""
-        Add(self.parser, "web", web)
-
-        # send_email.py
-        Add(self.parser, "subject", "OMERO - %(name)s was upgraded")
-        Add(self.parser, "sender", "sysadmin@openmicroscopy.org")
-        Add(self.parser, "recipients",
-            "ome-nitpick@lists.openmicroscopy.org.uk",
-            help="Comma-separated list of recipients")
-        Add(self.parser, "server", "%(name)s (%(address)s)")
-        Add(self.parser, "smtp_server", "smtp.dundee.ac.uk")
-        Add(self.parser, "weburl", "http://%(address)s/omero/webclient/")
 
         Add(self.parser, "skipweb", "false")
         Add(self.parser, "skipdelete", "true")
@@ -414,12 +407,6 @@ class InstallBaseCommand(Command):
         else:
             UnixInstall(self.NAME, args)
 
-        if "false" == args.skipemail.lower():
-            artifacts = Artifacts(args)
-            Email(artifacts, args)
-        else:
-            log.info("Skipping email")
-
 
 class InstallCommand(InstallBaseCommand):
     """
